define(['jquery', 'underscore', 'backbone', 'events', 'context', 'models/table/meta', 'tpls'],
    function ($, _, Backbone, Events, Context, Meta, Templates) {
      return Backbone.View.extend({
        tagName: "li",
        className: "table-list-item",
        template: Templates["sidebar/table"],
        metaTemplate: Templates["sidebar/meta"],
        enableSuccessEvents: true,
        disableMetaCache: false,
        deleteConfirm: "Are you sure you want to delete this table?",
        $dropdown: null,
        initialize: function () {
          _.bindAll(this, "setMeta", "setMetaTemplate", "deleteTable", "renameTable", "shortcut");
          this.listenTo(this.model, "change", this.render, this);
          this.listenTo(this.model, "remove", this.remove, this);
          this.listenTo(Events, "schema:changed", this.schemaChanged, this);
        },
        render: function () {
          this.$el.html(this.template(this.model.toJSON()));

          // bind dropdown
          this.$dropdown = this.$(".dropdown-menu");

          // manually bind events, because context menu gets detached
          this.$("a.table-delete").click(this.deleteTable);
          this.$("a.table-rename").click(this.renameTable);

          return this;
        },
        events: {
          "click a.table-item": "activate",
          "click a.table-settings": "settings",
          "click a.table-raw, a.table-filter": "shortcut"
        },
        shortcut: function (e) {
          if (e) {
            e.preventDefault();
          }

          var mode = $(e.currentTarget).data("mode");
          var model = this.model;
          var table = this.model.get("title");

          if (mode) {

            // 1. change query mode view without triggering query execute
            Events.trigger("table:shortcut", mode);
            // 2. change table - it will trigger query execute in filter view
            // 3. trigger default query execute in raw view after meta fetch
            this.changeTable(function () {
              Events.trigger("table:shortcut:post", model);
            }); // async, fetches table meta

          } else {
            console.warn("query mode missing, can't load table")
          }
        },
        schemaChanged: function () {
          // if this is the currently active table, update meta
          if (this.model.get("title") === Context.table) {
            this.setMeta(true);
          }
        },
        openBuilder: function (e) {
          e.stopPropagation();
          e.preventDefault();

          Context.section = "builder";
          Backbone.history.syncURL();

          this.enableSuccessEvents = false;
          this.changeTable();
        },
        highlightActive: function () {
          return this.$el.addClass("active").siblings().removeClass("active");
        },
        setMeta: function (force, cbk) {
          if (this.$el.next().is(".table-meta") && this.meta && !this.disableMetaCache && !force) {
            // set meta from cache
            Context.meta = this.meta.toJSON();
            this.$el.siblings(".table-meta").hide();
            this.$el.next().show();
            this.triggerSuccessEvents(cbk);
            console.log("load table meta from cache", this.meta.toJSON(), this.$el.next());
          } else {
            this.meta = new Meta();
            var that = this;
            this.meta.fetch({success: function (meta) {
              that.setMetaTemplate(meta, cbk);
            }});
          }
        },
        setMetaTemplate: function (meta, cbk) {

          this.$el.siblings(".table-meta").hide();

          // store meta in context
          Context.meta = meta.toJSON();

          this.$el.after(this.metaTemplate(meta.toJSON()));

          console.log("fetch table meta from server", meta.toJSON(), this.$el.next());

          // trigger global events only on successful meta retrieval
          this.triggerSuccessEvents(cbk);
        },
        triggerSuccessEvents: function (cbk) {
          Events.trigger("table:change", this.model, this.meta);
          if (this.enableSuccessEvents && Context.query_mode !== "raw") {
            Events.trigger("query:execute");
          }

          if (cbk && _.isFunction(cbk)) {
            cbk(this.model, this.meta);
          }
        },
        settings: function (e) {
          e.preventDefault();
        },
        deleteTable: function (e) {
          e.preventDefault();

          if (this.$dropdown) {
            this.$dropdown.hide();
          }

          if (confirm(this.deleteConfirm)) {
            var title = this.model.get("title");

            this.model.destroy({
              success: function () {

                // update context tables
                if (Context.tables) {
                  Context.tables = _.reject(Context.tables, function (table) {
                    return table === title;
                  });
                  console.log(Context.tables);
                }

                Events.trigger("table:deleted", title);
              },
              error: function () {
                alert("Unexpected error, table wasn't deleted.");
              }
            });
          }
        },
        renameTable: function (e) {
          e.preventDefault();

          // modal handled in sidebar view
          Events.trigger("table:rename", false, this.model);
        },
        activate: function (e) {
          e.preventDefault();

          Context.section = "main";
          Backbone.history.syncURL();

          this.enableSuccessEvents = true;
          this.changeTable();
        },
        changeTable: function (cbk) {
          Events.trigger("table:activate");

          var prevTable = Context.table;

          // set active table in global context
          Context.table = this.model.get("title");

          // set table meta
          this.setMeta(false, cbk);

          // sync url on table change
          if (this.model.get("title") !== prevTable) {
            Backbone.history.syncURL();
          }

          // highlight active item
          this.highlightActive();
        }
      });
    });
